;
;****************************************************************************
;
;  Purpose:
;	Provide Support For The Dallas DS1820 Temperature Sensor
;
;  Date:
;	11/21/93
;
;  Author:
;	John C. Wren
;
;  Modifications:
;	None
;
;  Processor:
;	Generic 8031
;
;  Assembler:
;	Avocet AVA51
;
;  Dependencies:
;	None
;
;  Files:
;	MACROS.INC
;
;  Comments:
;	This Code Is Designed To Manage Up To 8 Temperature Sensors, All On
;	A Single Port.  When The Initialization Routine Is Called, It 
;	Determines The Number Of Sensors Connected.  If A Port Pin Has No
;	Sensor Connected, Then The Mask That Specifies What Pins Are To Be
;	Checked Is Marked As Not To Check That Pin.  
;
;	If A Sensor Is Marked As Present, And Later Does Not Respond, It Is
;	Up To The User To Remove The Non-Responding Part From The Mask Of
;	Pins To Check.
;
;	If A Bad Temperature Is Read (Which Could Happen From Noise On The
;	Line, Etc), The Calling Routine Is Informed, But No Action Is Taken.
;
;	At Initialization Time, All Connected Sensors Are Issued A Start
;	Conversion Command.  After The Temperature Is Read (And Enough Time
;	Must Pass To Allow The Conversion Is Complete Before Reading, Etc)
;	A Start Conversion Command Is Then Issued For The Next Time The
;	Temperature Is Read.  The Implication Of This Is That If The Sensors
;	Are Issued A Start Conversion, And The Temperature Is Not Read For
;	A Great Length Of Time, The Actual Temperature Might Have Changed,
;	And The Sensor Will Have The Old Value.
;
;	Because Of The Nature Of The Dallas Parts, And The Timing Involved,
;	The MDELAY Macro In MACROS.INC Will Need To Be Edited For The Target
;	System.  Currently, The Macro Is Built For An 8031 Running At
;	11.0592MHz.  Also, Be Sure To Check The Value For The Loop In
;	DS1820_RESET.  This Needs To Be Adjusted To Wait For At Least 500
;	Microseconds For The Part To Respond.
;
;	The Byte Write And Read Bit Routines (DS1820_WRITE, And DS1820_
;	READ_BIT) Make The Presumption That The Are Allowed To Disable
;	Global Interrupts While Writing A Byte, Or Reading A Bit.  They
;	Also Turn It Back On When They Exit.  This Could Cause A Problem
;	For Code That Expects Interrupts To Not Be On While Calling These
;	Routines.
;
;  Philosophic:
;	This Code Is Dependent On Processors Speeds.  The Dallas Parts Have
;	A Method Of Reading Them That Takes Both Too Much And Too Little Time.
;	The Take Too Much Time In That The Read And Write Time Slots Are
;	Too Long For Instruction Looping To Waste Time, And Too Short To
;	Connected A Service Routine To Timer To Time Slice Read Them.
;
;	This Code Uses Instruction Looping To Waste Time, Because The 8031
;	I Was Using Was Being Clocked At 11.0592MHz.  On A Fast Processor,
;	Or A Task That Requires Other Things To Happen While The Values Are
;	Being Read From The Sensor, Using A Timer Interrupt May Be More
;	Viable.
;
;	The Dallas Parts Provide A Near Method Of Determing What And How Many
;	Devices Exist On The One Wire Bus.  In Most Cases, This Is Not Very
;	Useful.  If A System Starts Up And Interrogates The Parts Present,
;	Unless A Database Exists That Cross References The Sensor ID To A
;	Location, You Don't Know The Location Sensor That Just Reported An 
;	Over Temperature Condition.  The One Wire Bus Is Entirely Dependant
;	On Adding One Device At A Time, And Having A History For Each Part
;	(Presuming That You Care Where The Sensors Are Located, And Not Just
;	Some General Condition). 
;
;	In Addition, The Method For Determing The Sensors Present Requires
;	A Fair Amount Of Recursion.  This Takes Memory Which Many Of The
;	Small Systems Don't Have.  This Leaves Us With The Basic Method
;	Of One Sensor Per I/O Pin, Which While Not As Elegant As It Could
;	Be Gets The Job Done The Fasted, And With The Least Amount Of Over-
;	Head.  If You Do Implement A Sensor Search That Can Handle A Number
;	Of Sensors, I'd Love To Get A Copy.
; 
;****************************************************************************
;
;  Includes
;
		%include "macros.inc"
		seg	code
;
;****************************************************************************
;
;  Publics
;
		public	DS1820_INIT
		public	DS1820_GETCOUNT
		public	DS1820_PROCESS
		public	DS1820_READTEMP
		public	DS1820_CONVERT
;
;****************************************************************************
;
;  Equates
;
E_CMD_READROM	equ	033h			; Read ROM
E_CMD_MATCHROM	equ	055h			; Match ROM
E_CMD_SKIPROM	equ	0cch			; Skip ROM
E_CMD_SEARCHROM	equ	0f0h			; Search ROM
E_CMD_ALARMSRCH	equ	0ech			; Alarm Search
E_CMD_WRITESPM	equ	04eh			; Write Scratchpad Memory
E_CMD_READSPM	equ	0beh			; Read Scratchpad Memory
E_CMD_COPYSPM	equ	048h			; Copy Scrathpad Memory To EE
E_CMD_RECALLEE	equ	0b8h			; Recall EE
E_CMD_READPS	equ	0b4h			; Read Power Supply
E_CMD_CONVERT	equ	044h			; Convert T
;
E_SPM_TEMPL	equ	000h			; Temperature LSB
E_SPM_TEMPH	equ	001h			; Temperature MSB
E_SPM_TH	equ	002h			; TH/User Byte 1
E_SPM_TL	equ	003h			; TL/User Byte 2
E_SPM_RSVD1	equ	004h			; Reserved 1
E_SPM_RSVD2	equ	005h			; Reserved 2
E_SPM_CNTRMN	equ	006h			; Count Remain
E_SPM_CNTPERC	equ	007h			; Count Per C
E_SPM_CRC	equ	008h			; CRC
;
;****************************************************************************
;
;  Description:
;	Initialize The Sensors
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc, B, R0
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Sensors To Poll Later Are Determined By This Routine, And Stored
;	Into A Mask.  All Sensors That Are Detected Are Issued A Start
;	Conversion Command.
;
DS1820_INIT	proc
		mov	D_DS1820_PRSNT,#0	; Clear Installed Sensors
		mov	b,#1			; Start With Sensor 1
		mov	r0,#8			; Expected Number Of Sensors
;
l?p1		mov	D_DS1820_SLCT,b		; Set Select Mask
		call	DS1820_HI		; Temp Sensor 0 High
		call	DS1820_CONVERT		; Start Conversion
		jnc	l?p2			; If No Part, Skip
;
		mov	a,b			; Get Sensor Mask
		orl	D_DS1820_PRSNT,a	; Set Bit For Present Sensor
;
l?p2		mov	a,b			; Get Sensor Mask
		rl	a			; Next Sensor
		mov	b,a			; Store It Back
		djnz	r0,l?p1			; Repeat For All 8
;
;  Calculate The First Sensor To Use
;
		mov	a,D_DS1820_PRSNT	; Get Installed Sensor Map
		jz	l?p5			; If None Installed, Skip
;
		mov	r0,#8			; Maximum 8 Sensors
		mov	b,#1			; Start With Lowest Sensor
l?p3		mov	a,D_DS1820_PRSNT	; Get Installed Sensor Map
		anl	a,b			; AND Bit
		jnz	l?p4			; If Bit Set, Sensor Installed
		mov	a,b			; Get Mask
		rl	a			; Shift Left
		mov	b,a			; Move Back
		djnz	r0,l?p3			; Repeat Up To 8 Times
		jmp	l?p5			; Exit, No Installed Devices
;
;  'B' Contains The Bit Position Of The First Installed Sensor
;
l?p4		mov	D_DS1820_SLCT,b		; Store To Select Byte
;
l?p5		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Returns Number Of Installed Sensors
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	Acc Has Number Of Installed Sensors
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Sensors Must Have Been Previously Initialized
;
DS1820_GETCOUNT	proc
		push	0			; Save R0
		push	b			; Save B
		mov	a,D_DS1820_PRSNT	; Get Mask Of Installed Sensors
		mov	b,#0			; Clear Counter
l?p1		rrc	a			; Move Mask Right
		jnc	l?p2			; If Not Installed, Skip
		inc	b			; Increment Installed Count
l?p2		djnz	r0,l?p1			; Repeat For All 8 Bits
		mov	a,b			; Get Count To Acc
		pop	b			; Recover B
		pop	0			; Recover R0
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Read The Temperature From The Next Sensor
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY == 1 If No Sensors To Read From
;	CY == 0 If Sensor Was Read
;	  ACC != 0 If Temperature Invalid (Not In Range Supported By Part)
;	  ACC == 0 If Temperature Valid
;	    B == Sensor Number (0..7)
;	    DPTR == Temperature Read From Device
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc, B, DPTR
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
DS1820_PROCESS	proc
		mov	a,D_DS1820_PRSNT	; Get Installed Sensor Map
		jnz	l?p1			; If No Sensors, Exit
		setb	c			; No Installed Sensors
		ret				; Return To Caller
;
;  Read Temperature Into DPTR
;
l?p1		push	0			; Save R0
		call	DS1820_READTEMP		; Read The Temperature
		jnc	l?p1z			; If OK, Skip
		pop	0			; Recover R0
		ret				; Return To Caller
;
;  Now Make Sure Temperature Legal, Acc Will Tell If Good Or Not
;
l?p1z		call	DS1820_RANGECHECK	; Make Sure A Legal Value
;
;  Now Determine Temperature Sensor Number
;
		push	acc			; Save Return Value
		mov	a,D_DS1820_SLCT		; Get Currently Selected Sensor
		mov	b,#0			; Start With 0
l?p1a		rrc	a			; Shift Right
		jc	l?p1b			; If Set, We Found It
		inc	b			; Increment Sensor Number
		jmp	l?p1a			; Repeat Until Set Bit Found
;
;  Calculate Next Sensor To Poll
;
l?p1b		push	b			; Save B
		mov	r0,#8			; Maximum 8 Sensors
		mov	a,D_DS1820_SLCT		; Start With Current Sensor
		rl	a			; Move To Next Sensor
		mov	b,a			; Store To B
l?p2		mov	a,D_DS1820_PRSNT	; Get Installed Sensor Map
		anl	a,b			; AND Bit
		jnz	l?p3			; If Bit Set, Sensor Installed
		mov	a,b			; Get Mask
		rl	a			; Shift Left
		mov	b,a			; Move Back
		djnz	r0,l?p2			; Repeat Up To 8 Times
;
l?p3		mov	D_DS1820_SLCT,b		; Store New Selected Sensor
		pop	b			; Recover B
		pop	acc			; Recover Return Value
		pop	0			; Recover R0
		clr	c			; Acc Has Status, DPTR Has Temp
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Validates Temperature In -55C To +125C Range
;
;  Entry Requirements:
;	DPTR Has Temperature From Sensor
;
;  On Exit:
;	Acc == 0 If Value In DPTR In Range
;	Acc != 0 If Valus In DPTR Not In Range
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	0 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	0xff92 Is -55C, 0x00fa Is +125C
;
DS1820_RANGECHECK proc
		mov	a,dph			; Get High Side
		cjeq	a,#0,l?p1		; If 0, Check Positive Temps
		cjeq	a,#-1,l?p2		; If -1, Check Negative Temps
		jmp	l?p3			; Error, Exit
;
;  Check The Positive Temperature Range
;
l?p1		mov	a,dpl			; Get Low Side
		cjgt	a,#0fah,l?p3		; If > +125C, Error
		clr	a			; No Error
		ret				; Return To Caller
;
;  Check The Negative Temperature Range
;
l?p2		mov	a,dpl			; Get Low Side
		cjlt	a,#092h,l?p3		; If < -55C, Error
		clr	a			; No Error
		ret				; Return To Caller
;
;  Error Return
;
l?p3		mov	a,#-1			; Error
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Performs Read From Selected Sensor
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	DPTR Has Temperature Bytes
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc, DPTR
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	The DS1820, et al Returns 9 Bytes.  The Remaining 7 Are Scratchpads,
;	Some Other Stuff.  While The Read Can Be Aborted Midstream By 
;	Issuing A Reset, The Bytes Are Read In Case They Might Be Needed
;	For Some Other Purpose.
;
DS1820_READTEMP	proc
		call	DS1820_RESET		; Send Reset
		jc	l?p1			; If Response, Skip
		setb	c			; CY == 1 Means Error
		ret				; Return To Caller
;
l?p1		mov	a,#E_CMD_SKIPROM	; Skip ROM Command
		call	DS1820_WRITE		; Write A Byte
		mov	a,#E_CMD_READSPM	; Read Scratchpad Memory
		call	DS1820_WRITE		; Write A Byte
;
;  Now Read The Bytes From The Part
;
		call	DS1820_READ		; Read The Byte
		mov	dpl,a			; Store Low Of Temperature
		call	DS1820_READ		; Read The Byte
		mov	dph,a			; Store High Of Temperature
;
;  Toss Remaining Bytes
;
		mov	r0,#7			; Read Remaining 7 Bytes
l?p2		call	DS1820_READ		; Read And Toss
		djnz	r0,l?p2			; Repeat R0 Times
		clr	c			; CY == 1 Means OK
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Issues Start Conversion Command To Selected Sensor
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY == 0 If Sensor Did Not Respond To Reset
;	CY == 1 If Sensor Present
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
DS1820_CONVERT	proc
		push	acc			; Save Acc
		call	DS1820_RESET		; Send Reset
		jnc	l?p1			; If No Response
;
;  Now Skip ROM And Issue Convert Command
;
		mov	a,#E_CMD_SKIPROM	; Skip ROM Command
		call	DS1820_WRITE		; Write A Byte
		mov	a,#E_CMD_CONVERT	; Start Conversion
		call	DS1820_WRITE		; Write A Byte
		setb	c			; Part Is Present
;
l?p1		pop	acc			; Recover Acc
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Issues Reset To Selected Sensor
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY == 0 If Device Not Present
;	CY == 1 If Device Present
;
;  Affected:
;	PSW.CY, PSW.F0, PSW.Z, PSW.P, Acc
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	If Running On More Than A 12Mhz Processor, Be Sure To Adjust The
;	Amount Of Time The Loop Waits For The Part To Assert The Presence
;	Pulse.  Need To Wait At Least 500us.  Times To The Side Are For A
;	12MHz Processor Clock, On A Generic 8031.
;
DS1820_RESET	proc
		push	0			; Save R0
		call	DS1820_LO		; Sensor Low
		call	DS1820_SLEEP500		; Sleep 500us
		call	DS1820_HI		; Sensor High
		nop				; Delay For DS1820 To Go High
		nop				; Delay For DS1820 To Go High
		nop				; Delay For DS1820 To Go High
		nop				; Delay For DS1820 To Go High
		nop				; Delay For DS1820 To Go High
;
;  Now Wait For 500us For DS1820 To Assert Presence Pulse.   DS1820_GET_BIT
;  Takes A Minimum Of 3.33us To Execute.
;
		mov	r0,#21			; 23.87us * 21 = 506us
		clr	f0			; Clear Carry
l?p1		call	DS1820_RD		; Read Sensor		13us
		cpl	c			; Complement It		 1us
		orl	c,f0			; OR In F0		 2us
		jc	l?p2			; Leave When Found	 2us
		mov	f0,c			; Move C To F0		 2us
		djnz	r0,l?p1			; Repeat R0 Times	 2us
;
		pop	0			; Recover R0
		ret				; Return To Caller
;
;  Return CY == 1 After DS1820 Done Asserting Presence Pulse
;
l?p2		call	DS1820_RD		; Read Sensor
		jnc	l?p2			; While Asserting, Loop
;
		pop	0			; Recover R0
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Write Byte To Currently Selected Sensor
;
;  Entry Requirements:
;	Acc Has Value To Write To Sensor
;
;  On Exit:
;	None
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Global Interrpts Are Disabled, And The Presumption Is Made That The
;	Routine May Turn Them Back On When It Exits.
;
DS1820_WRITE	proc
		clr	ea			; No Interrupts
		push	0			; Save R0
;
		mov	r0,#8			; 8 Bits
l?p1		rrc	a			; Shift LSB To Carry
		jc	l?p2			; If 1, Skip
		call	DS1820_WRITE_0		; Write A 0 Bit
		sjmp	l?p3			; Skip
;
l?p2		call	DS1820_WRITE_1		; Write A 1 Bit
;
l?p3		djnz	r0,l?p1			; Repeat R0 Times
;
		pop	0			; Recover R0
		setb	ea			; Allow Interrupts
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Write A 0 Bit To The Currently Selected Part
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
DS1820_WRITE_0	proc
		call	DS1820_LO		; Sensor Low
		MDELAY	60000			; 60000
		call	DS1820_HI		; Sensor High
		MDELAY	5000			; 5us
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Write A 1 Bit To The Currently Selected Part
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
DS1820_WRITE_1	proc
		call	DS1820_LO		; Sensor Low
		MDELAY	5000			; 5us From Low To High
		call	DS1820_HI		; Sensor High
		MDELAY	60000			; 60000
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Read A Byte From The Currently Selected Sensor
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	Acc Has Byte Read From Sensor
;
;  Affected:
;	PSW.CY, PSW.Z, PSW.P, Acc
;
;  Stack:
;	3 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	None
;
DS1820_READ	proc
		push	0			; Save R0
;
		mov	r0,#8			; Read 8 Bits
		clr	a			; Clear For Read
l?p1		call	DS1820_READ_BIT		; Get A Bit
		rrc	a			; Shift In
		djnz	r0,l?p1			; Do For 8 Bits
;
		pop	0			; Recover R0
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Read Bit From Currently Selected Sensor
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY Has 1 Or 0 As Value Of Bit
;
;  Affected:
;	Acc
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Global Interrpts Are Disabled, And The Presumption Is Made That The
;	Routine May Turn Them Back On When It Exits.
;
DS1820_READ_BIT	proc
		clr	ea			; No Interrupts
		push	acc			; Save Acc		2us
		mov	a,D_DS1820_SLCT		; Get Mask		1us
		xrl	a,#-1			; Make AND Mask		2us
		anl	p1,a			; Port Pin Low		1us
;		
		nop				; Delay			1us
		mov	a,D_DS1820_SLCT		; Get Mask		1us
		orl	p1,a			; Port Pin High		1us
;
		MDELAY	15000			; 15us
;		
		mov	a,p1			; Read Port		1us
		anl	a,D_DS1820_SLCT		; Keep Only Good Bit	1us
		clr	c			; In Case Acc==0	1us
		jz	l?p1			; If 0, Skip		2us
		setb	c			; If Acc <> 0, CY == 1	1us
;
l?p1		MDELAY	60000			; 50us
		pop	acc			; Recover Acc
		setb	ea			; Allow Interrupts
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Drive Output Line To Currently Selected Sensor High
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	1 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	2+1+1+2+2 (8) Cycles
;
DS1820_HI	proc
		push	acc			; Save Acc
		mov	a,D_DS1820_SLCT		; Get Mask
		orl	p1,a			; Set Bit
		pop	acc			; Recover Acc
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Drive Output Line To Currently Selected Sensor Low
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	1 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	2+1+1+1+2+2 (9) Cycles
;
DS1820_LO	proc
		push	acc			; Save Acc
		mov	a,D_DS1820_SLCT		; Get Mask
		xrl	a,#-1			; Make AND Mask
		anl	p1,a			; Set Bit
		pop	acc			; Recover Acc
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Sample Bit From The Currently Selected Sensor
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	CY Has 1 Or 0 For Current Bit Value
;
;  Affected:
;	PSW.CY
;
;  Stack:
;	1 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	2+1+1+2+1+2+2 (11) Cycles
;
DS1820_RD	proc
		push	acc			; Save Acc
		mov	a,p1			; Read Port
		anl	a,D_DS1820_SLCT		; Keep Only Good Bit
		clr	c			; In Case Acc==0
		jz	l?p1			; If 0, Skip
		setb	c			; If Acc <> 0, CY == 1
l?p1		pop	acc			; Recover Acc
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Description:
;	Delay 500 Microseconds
;
;  Entry Requirements:
;	None
;
;  On Exit:
;	None
;
;  Affected:
;	None
;
;  Stack:
;	2 Bytes, Not Including Space Used By Called Routines
;
;  Comments:
;	Make Sure MDELAY Macro Has Been Adjusted For Target System
;
DS1820_SLEEP500	proc
		push	0			; Save R0
		mov	r0,#10			; 361ns
l?p1		MDELAY	50000			; 50us
		djnz	r0,l?p1			; 500.61+ us
		pop	0			; Recover R0
		ret				; Return To Caller
		endproc
;
;****************************************************************************
;
;  Data Area
;
		seg	data
D_DS1820_PRSNT	ds	1
D_DS1820_SLCT	ds	1
;
;****************************************************************************
;
		end
